/* This file is part of the KDE project
 * SPDX-FileCopyrightText: 2006, 2010 Thomas Zander <zander@kde.org>
 *
 * SPDX-License-Identifier: LGPL-2.0-or-later
 */

#include "KoShapeConfigFactoryBase.h"

KoShapeConfigFactoryBase::KoShapeConfigFactoryBase() = default;

KoShapeConfigFactoryBase::~KoShapeConfigFactoryBase() = default;
