/* This file is part of the KDE project
 * SPDX-FileCopyrightText: 2014 Arjen Hiemstra <ahiemstra@heimr.nl>
 *
 * SPDX-License-Identifier: GPL-2.0-or-later
 */

// font size is dependent on the line height of the application
// pixel size is the pixel size of the font on a 1080p monitor, and adjusted for
// other resolutions

var values = {
    application: {
        family: "Source Sans Pro",
        style: "Regular",
        size: 1,
    },
    applicationSemi: {
        family: "Source Sans Pro",
        style: "Semibold",
        pixelSize: 18,
    },
    applicationLight: {
        family: "Source Sans Pro",
        style: "Light",
        pixelSize: 18,
    },
    optionsMenu: {
        family: "Source Sans Pro",
        style: "Semibold",
        pixelSize: 22,
    },
    small: {
        family: "Source Sans Pro",
        style: "Bold",
        size: 0.75,
    },
    pageHeader: {
        family: "Source Sans Pro",
        style: "Light",
        pixelSize: 38,
    },
    welcomeHeader: {
        family: "Source Sans Pro",
        style: "Semibold",
        pixelSize: 30,
    },
    welcomeSidebar: {
        family: "Source Sans Pro",
        style: "Regular",
        pixelSize: 24,
    },
    presentationTime: {
        family: "Source Sans Pro",
        style: "Extralight",
        pixelSize: 120,
    },
    presentationFx: {
        family: "Source Sans Pro",
        style: "Regular",
        pixelSize: 24,
    },
    filelistheader: {
        family: "Source Sans Pro",
        style: "Semibold",
        pixelSize: 18,
    },
    templateLabel: {
        family: "Source Sans Pro",
        style: "Light",
        pixelSize: 24
    },
    title: {
        family: "Source Sans Pro",
        style: "Light",
        pixelSize: 38,
    },
    panelHandle: {
        family: "Source Sans Pro",
        style: "Regular",
        size: 1.25,
    },
    panelHeader: {
        family: "Source Sans Pro",
        style: "Regular",
        size: 1.5,
    },
    panelSection: {
        family: "Source Sans Pro",
        style: "Regular",
        size: 1.25,
    },
    toolbar: {
        family: "Source Sans Pro",
        style: "Semibold",
        pixelSize: 24,
    }

}
