/* This file is part of the KDE project
 *
 * SPDX-FileCopyrightText: 2006, 2009-2010 Thomas Zander <zander@kde.org>
 *
 * SPDX-License-Identifier: LGPL-2.0-or-later
 */

#include "KoDataCenterBase.h"

KoDataCenterBase::KoDataCenterBase() = default;

KoDataCenterBase::~KoDataCenterBase() = default;
