/* This file is part of the KDE project
 * SPDX-FileCopyrightText: 2007 Jan Hambrecht <jaham@gmx.net>
 *
 * SPDX-License-Identifier: LGPL-2.0-or-later
 */

#ifndef _KARBON_H_
#define _KARBON_H_

#include "KoCanvasResourceManager.h"

namespace Karbon
{

enum KarbonCanvasResource {
    FirstKarbonResource = KoCanvasResourceManager::KarbonStart,
};
}

#endif // _KARBON_H_
