/*
   SPDX-FileCopyrightText: 2018 Damien Caliste <dcaliste@free.fr>

   SPDX-License-Identifier: LGPL-2.0-or-later
*/

#ifndef XLSXUTILS_H
#define XLSXUTILS_H

#include <QDebug>
#include <QLoggingCategory>

Q_DECLARE_LOGGING_CATEGORY(lcXlsxImport)

#endif
