/* This file is part of the KDE project
   SPDX-FileCopyrightText: 2008 Thorsten Zachmann <zachmann@kde.org>

   SPDX-License-Identifier: LGPL-2.0-or-later
*/

#include "KPrEventActionWidget.h"

KPrEventActionWidget::KPrEventActionWidget(QWidget *parent)
    : QWidget(parent)
{
}

KPrEventActionWidget::~KPrEventActionWidget() = default;
