/*
 *  SPDX-FileCopyrightText: 2001 Graham Short. <grahshrt@netscape.net>
 *
 *  SPDX-License-Identifier: GPL-2.0-or-later
 */

#ifndef QPRO_COMMON_H
#define QPRO_COMMON_H

#ifdef QP_TRACE
#define QP_DEBUG(x) cerr << x
#else
#define QP_DEBUG(x)
#endif

#endif // QPRO_COMMON_H
