/* This file is part of the KDE project
   SPDX-FileCopyrightText: 1999-2006 The KSpread Team <calligra-devel@kde.org>
   SPDX-FileCopyrightText: 2010 Marijn Kruisselbrink <mkruisselbrink@kde.org>

   SPDX-License-Identifier: LGPL-2.0-or-later
*/

#include "CellEditorBase.h"

using namespace Calligra::Sheets;

CellEditorBase::CellEditorBase() = default;

CellEditorBase::~CellEditorBase() = default;
