/*
 *  SPDX-FileCopyrightText: 2010 Carlos Licea <carlos@kdab.com>
 *
 *  SPDX-License-Identifier: LGPL-2.1-or-later
 */

#include "KoCellChild.h"

KoCellChild::KoCellChild() = default;

KoCellChild::~KoCellChild() = default;
