/* This file is part of the KDE project
 * SPDX-FileCopyrightText: 2011 Paul Mendez <paulestebanms@gmail.com>
 *
 * SPDX-License-Identifier: LGPL-2.0-or-later
 */

#ifndef TESTDELCUSTOMSLIDESHOWCOMMAND_H
#define TESTDELCUSTOMSLIDESHOWCOMMAND_H

#include <QObject>

class TestDelCustomSlideShowCommand : public QObject
{
    Q_OBJECT
private Q_SLOTS:
    void delCustomSlideShow();
};

#endif // TESTDELCUSTOMSLIDESHOWCOMMAND_H
