/* This file is part of the KDE project
 *
 * SPDX-FileCopyrightText: 2012 Inge Wallin <inge@lysator.liu.se>
 *
 * SPDX-License-Identifier: LGPL-2.0-or-later
 */

#ifndef THREED_UTILS_H
#define THREED_UTILS_H

// Qt
#include <QVector3D>

QVector3D odfToVector3D(QString &string);

#endif
