/* Swinder - Portable library for spreadsheet
   SPDX-FileCopyrightText: 2009 Marijn Kruisselbrink <mkruisselbrink@kde.org>

   SPDX-License-Identifier: LGPL-2.0-or-later
 */
#include "substreamhandler.h"

namespace Swinder
{

SubStreamHandler::~SubStreamHandler() = default;

} // namespace Swinder
