/* This file is part of the KDE project
 * SPDX-FileCopyrightText: 2006 Thomas Zander <zander@kde.org>
 *
 * SPDX-License-Identifier: LGPL-2.0-or-later
 */

#ifndef WORDS_EXPORT_H_WRAPPER
#define WORDS_EXPORT_H_WRAPPER

/* This is the one which does the actual work, generated by cmake */
#include "words_generated_export.h"

/* Now the same for WORDS_TEST_EXPORT, if compiling with unit tests enabled */
#ifdef COMPILING_TESTS
#define WORDS_TEST_EXPORT WORDS_EXPORT
#else /* not compiling tests */
#define WORDS_TEST_EXPORT
#endif

#endif
