/* This file is part of the KDE project
   SPDX-FileCopyrightText: 2016 Friedrich W. H. Kossebau <kossebau@kde.org>

   SPDX-License-Identifier: LGPL-2.0-or-later
*/

#include <OkularOdtGenerator.h>

OKULAR_EXPORT_PLUGIN(OkularOdtGenerator, "libokularGenerator_odt.json")

#include "OkularOdtGeneratorPlugin.moc"
