/* This file is part of the KDE project
   SPDX-FileCopyrightText: 2009 Thorsten Zachmann <zachmann@kde.org>

   SPDX-License-Identifier: LGPL-2.0-or-later
*/

#include "KoLoadingShapeUpdater.h"

KoLoadingShapeUpdater::KoLoadingShapeUpdater() = default;

KoLoadingShapeUpdater::~KoLoadingShapeUpdater() = default;
