/* This file is part of the KDE project
 * SPDX-FileCopyrightText: 2011 Paul Mendez <paulestebanms@gmail.com>
 *
 * SPDX-License-Identifier: LGPL-2.0-or-later
 */

#ifndef TESTADDCUSTOMSLIDESHOWCOMMAND_H
#define TESTADDCUSTOMSLIDESHOWCOMMAND_H

#include <QObject>

class TestAddCustomSlideShowCommand : public QObject
{
    Q_OBJECT
private Q_SLOTS:
    void addCustomSlideShow();
};

#endif // TESTADDCUSTOMSLIDESHOWCOMMAND_H
