/*
 *  SPDX-FileCopyrightText: 2009 Cyrille Berger <cberger@cberger.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation;
 * either version 2, or (at your option) any later version of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#ifndef _SECTION_H_
#define _SECTION_H_

#include <QMetaType>
#include <QString>

#include "SectionGroup.h"

class Layout;
class SectionContainer;
class RootSection;

class Section : public SectionGroup
{
public:
    explicit Section(RootSection *_rootSection);
    Section(const Section &_rhs);
    virtual ~Section();

public:
    SectionContainer *sectionContainer();
    const QString &name() const;
    void setName(const QString &_name);
    Layout *layout();
    void setLayout(Layout *layout);

private:
    Layout *m_layout;
    SectionContainer *m_sectionContainer;
    QString m_name;
};

Q_DECLARE_METATYPE(Section *)

#endif
