/* This file is part of the KDE project
   SPDX-FileCopyrightText: 2006, 2008 Jan Hambrecht <jaham@gmx.net>
   SPDX-FileCopyrightText: 2007 David Faure <faure@kde.org>
   SPDX-FileCopyrightText: 2007 Dirk Mueller <mueller@kde.org>
   SPDX-FileCopyrightText: 2008 Patrick Spendrin <ps_ml@gmx.de>

   SPDX-License-Identifier: LGPL-2.0-or-later
*/

#ifndef KARBONCURSOR_H
#define KARBONCURSOR_H

class QCursor;

namespace KarbonCursor
{
/** needle arraow cursor */
QCursor needleArrow();

/** needle arrow with four way arrow */
QCursor needleMoveArrow();
};

#endif // KARBONCURSOR_H
