/* This file is part of the KDE project
 * SPDX-FileCopyrightText: 2011 Boudewijn Rempt <boud@valdyas.org>
 *
 * SPDX-License-Identifier: LGPL-2.0-or-later
 */

#include "KoStartupDialog.h"

KoStartupDialog::KoStartupDialog(QWidget *parent)
    : QDialog(parent)
{
}
