/*
 *  SPDX-FileCopyrightText: 2015 Dmitry Kazakov <dimula73@gmail.com>
 *
 *  SPDX-License-Identifier: GPL-2.0-or-later
 */

#include "kundo2commandextradata.h"

KUndo2CommandExtraData::~KUndo2CommandExtraData() = default;
