/* This file is part of the KDE project
 * SPDX-FileCopyrightText: 2008 Jan Hambrecht <jaham@gmx.net>
 *
 * SPDX-License-Identifier: LGPL-2.0-or-later
 */

#ifndef KoShapeBackgroundPrivate_H
#define KoShapeBackgroundPrivate_H

class KoShapeBackgroundPrivate
{
public:
    KoShapeBackgroundPrivate();
    virtual ~KoShapeBackgroundPrivate();
};

#endif
