/* This file is part of the KDE project
 * SPDX-FileCopyrightText: 2011 C. Boemann <cbo@kogmbh.com>
 *
 * SPDX-License-Identifier: LGPL-2.0-or-later
 */

#include "KoTextLayoutRootAreaProvider.h"

KoTextLayoutRootAreaProvider::KoTextLayoutRootAreaProvider() = default;

KoTextLayoutRootAreaProvider::~KoTextLayoutRootAreaProvider() = default;
