/* This file is part of the KDE project
 * SPDX-FileCopyrightText: 2010 Carlos Licea <carlos@kdab.com>
 *
 * SPDX-License-Identifier: LGPL-2.0-or-later
 */

#ifndef GLOBALS_H
#define GLOBALS_H

#include <QPointF>
#include <QSizeF>

const QPointF initialsBoxPoint = QPointF(0.0, 0.0);
const QSizeF initialsBoxSize = QSizeF(20.0, 20.0);
const QPointF commentBoxPoint = QPointF(22.0, 0.0);
const QSizeF commentBoxSize = QSizeF(100.0, 100.0);

const QSizeF wholeSize = QSizeF(120.0, 100.0);

#endif
