/*  This file is part of the KDE project
    SPDX-FileCopyrightText: 2006 David Faure <faure@kde.org>
    SPDX-FileCopyrightText: 2006 Thomas Zander <zander@kde.org>

    SPDX-License-Identifier: LGPL-2.0-or-later
*/

#ifndef KOMAIN_EXPORT_H_WRAPPER
#define KOMAIN_EXPORT_H_WRAPPER

/* This is the one which does the actual work, generated by cmake */
#include "komain_generated_export.h"

/* Now the same for KOMAIN_TEST_EXPORT, if compiling with unit tests enabled */
#ifdef COMPILING_TESTS
#define KOMAIN_TEST_EXPORT KOMAIN_EXPORT
#else /* not compiling tests */
#define KOMAIN_TEST_EXPORT
#endif

#endif
