/* This file is part of the KDE project
 * SPDX-FileCopyrightText: 2013 Jos van den Oever <jos@vandenoever.info>
 *
 * SPDX-License-Identifier: LGPL-2.0-or-later
 */
#ifndef TESTPPT_H
#define TESTPPT_H

#include <QObject>

class TestPPT : public QObject
{
    Q_OBJECT

private Q_SLOTS:

    void testPPT();
};

#endif
