/*  This file is part of the KDE project
    SPDX-FileCopyrightText: 2006 David Faure <faure@kde.org>

    SPDX-License-Identifier: LGPL-2.0-or-later
*/

#ifndef KOPAGEAPP_EXPORT_H_WRAPPER
#define KOPAGEAPP_EXPORT_H_WRAPPER

/* This is the one which does the actual work, generated by cmake */
#include "kopageapp_generated_export.h"

#define KOPAGEAPP_TEST_EXPORT KOPAGEAPP_EXPORT

#endif
