// This file is part of the KDE project
// SPDX-FileCopyrightText: 2022 Tomas Mecir <mecirt@gmail.com>
// SPDX-FileCopyrightText: 2007 David Faure <faure@kde.org>
// SPDX-License-Identifier: LGPL-2.0-or-later

#ifndef SHEETS_ENGINE_EXPORT_H_WRAPPER
#define SHEETS_ENGINE_EXPORT_H_WRAPPER

/* This is the one which does the actual work, generated by cmake */
#include "sheets_engine_generated_export.h"

/* Now the same for CALLIGRA_SHEETS_ENGINE_TEST_EXPORT, if compiling with unit tests enabled */
#ifdef COMPILING_TESTS
#define CALLIGRA_SHEETS_ENGINE_TEST_EXPORT CALLIGRA_SHEETS_ENGINE_EXPORT
#else /* not compiling tests */
#define CALLIGRA_SHEETS_ENGINE_TEST_EXPORT
#endif

#endif
