/*
 * This file is part of Office 2007 Filters for Calligra
 *
 * SPDX-FileCopyrightText: 2010 Nokia Corporation and /or its subsidiary(-ies).
 *
 * Contact: Suresh Chande suresh.chande@nokia.com
 *
 * SPDX-License-Identifier: LGPL-2.1-only
 *
 */

#include <QString> // used in generated/generatedShapes.h, which is included
// inside a function in this file.

#include "PredefinedShapeHelper.h"

PredefinedShapeHelper::PredefinedShapeHelper()
{
#include "generated/generatedShapes.h"
}
