/* This file is part of the KDE project
  SPDX-FileCopyrightText: 2008 Pierre Stirnweiss <pstirnweiss@googlemail.com>

  SPDX-License-Identifier: LGPL-2.0-or-later
*/

#include "StyleManagerWelcome.h"

StyleManagerWelcome::StyleManagerWelcome(QWidget *parent)
    : QWidget(parent)
{
    widget.setupUi(this);
}
