/* This file is part of the KDE project
 * SPDX-FileCopyrightText: 2009 Jan Hambrecht <jaham@gmx.net>
 *
 * SPDX-License-Identifier: LGPL-2.1-or-later
 */

#include "KoFilterEffectConfigWidgetBase.h"

KoFilterEffectConfigWidgetBase::KoFilterEffectConfigWidgetBase(QWidget *parent)
    : QWidget(parent)
{
}
