/* This file is part of the KDE project

   @@COPYRIGHT@@

   SPDX-License-Identifier: LGPL-2.0-or-later
*/

// Own
#include "TestLoading.h"

// KoChart
#include "ChartShape.h"

using namespace KoChart;

TestLoading::TestLoading()
    : TestLoadingBase()
{
}

void TestLoading::testLabels()
{
    // Your code goes here
}

void TestLoading::testInternalTable()
{
    // Your code goes here
}

void TestLoading::testDataSets()
{
    // Your code goes here
}

void TestLoading::testLegend()
{
    // Your code goes here
}

QTEST_MAIN(TestLoading)
